# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - SharedUpdates Action

    @copyright: 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import ActionBase
from MoinMoin.Page import Page
from MoinMoin import wikiutil
from MoinShare import getPreferredOutputTypes, getOutputTypes, getUpdatedTime
from MoinSupport import escattr, formatText, get_form, getFragments, \
    getMetadata, getParserClass, getPathInfo, writeHeaders, ActionSupport

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

escape = wikiutil.escape

Dependencies = ['pages']

# Action class and supporting functions.

class SharedUpdates(ActionBase, ActionSupport):

    "An update manifest retrieval dialogue requesting various parameters."

    def get_form_html(self, buttons_html):
        _ = self._
        request = self.request
        form = self.get_form()
        page = self.page

        pagename = form.get("pagename", [page.page_name])[0]

        d = {
            "buttons_html"          : buttons_html,
            "pagename_label"        : escape(_("Page name for updates")),
            "pagename"              : escattr(pagename),
            }

        return '''
<table>
    <tr>
        <td class="label"><label>%(pagename_label)s</label></td>
        <td class="content">
            <input name="pagename" type="text" size="40" value="%(pagename)s" />
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>
''' % d

    def do_action(self):

        "Write the syndication resource."

        write_resource(self.request)
        return 1, None

    def render_success(self, msg, msgtype=None):

        """
        Render neither 'msg' nor 'msgtype' since a resource has already been
        produced.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

def write_resource(request):

    """
    For the given 'request', write an Atom summary of updates found on the
    specified page.
    See: http://tools.ietf.org/html/rfc4287
    """

    form = get_form(request)

    pagename = form.get("pagename", [request.page.page_name])[0]

    page = Page(request, pagename)
    metadata = getMetadata(page)

    # Output summary data...

    writeHeaders(request, "application/atom+xml", metadata)
    updated = getUpdatedTime(metadata)

    # Atom output...

    # Using the page name and the page URL in the title, link and
    # subtitle.

    path_info = getPathInfo(request)
    link = "%s%s" % (request.getBaseURL(), path_info)

    d = {
        "title"     : escape(path_info[1:]),
        "link"      : escape(link),
        "href"      : escattr(link),
        "updated"   : escape(updated),
        }

    # Start the output.

    request.write('''\
<?xml version="1.0" encoding="utf-8"?>\r
   <feed xmlns="http://www.w3.org/2005/Atom">\r
   <title type="text">%(title)s</title>\r
   <subtitle type="text">Shared updates published on %(link)s</subtitle>\r
   <link rel="self" href="%(href)s"/>\r
   <updated>%(updated)s</updated>\r
''' % d)

    # Get the fragment regions for the page.

    for n, (format, attributes, body) in enumerate(getFragments(page.get_raw_body())):

        # Produce a fragment identifier.
        # NOTE: Choose a more robust identifier where none is explicitly given.

        fragment = attributes.get("fragment", str(n))
        summary = attributes.get("summary", "Update #%d" % n)

        # Get the URL that yields only the fragment.

        fragment_link = "%s?action=SharedUpdate&fragment=%s" % (link, fragment)

        # Get the preferred content types available for the fragment.

        preferred = getPreferredOutputTypes(request, getOutputTypes(request, format))

        download_links = []

        for mimetype in preferred:
            specific_link = "%s&type=%s&doit=1" % (fragment_link, mimetype)

            download_links.append('<link rel="alternate" type="%s" href="%s"/>' % (
                escattr(mimetype), escattr(specific_link)))

        # Try and obtain some suitable content for the entry.
        # NOTE: Could potentially get a summary for the fragment.

        content = None

        if "text/html" in preferred:
            parser_cls = getParserClass(request, format)
            parser = parser_cls(body, request)

            if format == "html":
                content = body
            elif hasattr(parser, "formatForOutputType"):
                s = StringIO()
                parser.formatForOutputType("text/html", write=s.write)
                content = s.getvalue()
            else:
                fmt = request.html_formatter
                fmt.setPage(page)
                content = formatText(body, request, fmt, parser_cls)

        # NOTE: The published and updated details would need to be deduced from
        # NOTE: the page history instead of being taken for the page as a whole.

        d = {
            "title"             : escape(summary),
            "fragment_link"     : escape(fragment_link),
            "download_links"    : "\r\n".join(download_links),
            "content"           : content and ('<content type="html">%s</content>' % escape(content)) or "",
            "updated"           : escape(updated),
            }

        # Write the entry output.

        request.write('''\
<entry>\r
<title>%(title)s</title>\r
<id>%(fragment_link)s</id>\r
<updated>%(updated)s</updated>\r
%(download_links)s\r
%(content)s\r
</entry>\r
''' % d)

    # End the feed output.

    request.write('</feed>\r\n')

# Action function.

def execute(pagename, request):
    SharedUpdates(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
